/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.se.eip.patterns;

import java.net.URI;
import java.util.List;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.component.framework.Constants.MEPConstants;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.util.InputStreamForker;
import org.ow2.petals.se.eip.ExchangeContext;

/**
 * 
 * Dispatch the exchange to all Consumes services defined in the service unit.
 * <br>
 * <br>
 * - Accept InOnly and RobustInOnly <br>
 * - If a Fault or Error occurs on a call, stop the process and return the Fault
 * or Error Note : The exchange will be sent back automatically by the framework
 * 
 * @author alouis, chamerling - eBM WebSourcing
 * 
 */
public class Dispatcher extends AbstractPattern {

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.se.eip.patterns.Pattern#processPattern(org.ow2.petals.
     * component.framework.util.Exchange, org.ow2.petals.se.eip.ExchangeContext)
     */
    public void process(Exchange exchange, ExchangeContext context) throws PEtALSCDKException,
            MessagingException {
        Exchange originalExchange = exchange;
        List<Consumes> consumesList = context.getSUConsumes(originalExchange.getEndpoint());

        if (consumesList.size() == 0) {
            throw new PEtALSCDKException(
                    "The Service Unit must define at least 1 Consumes sections for the pattern "
                            + this.getPatternName());
        }

        sendAsyncExchanges(consumesList, exchange, context);
    }

    /**
     * Check InOnly or RobustInOnly pattern. <br>
     * 
     * @param uri
     * @return true if MEP is InOnly, false otherwise
     */
    protected boolean validateMEP(URI uri) {
        return MEPConstants.IN_ONLY_PATTERN.equals(uri);
    }

    /**
     * Send a copy of the exchange's IN message to all the services to consume.
     * If an error or a Fault occurred, it is reported to the original exchange.
     * 
     * @param consumesList
     * @param exchange
     * @param context
     * @throws JBIException
     * @throws PEtALSCDKException
     */
    protected void sendAsyncExchanges(List<Consumes> consumesList, Exchange exchange,
            ExchangeContext context) throws PEtALSCDKException, MessagingException {

        NormalizedMessage inMessage = exchange.getInMessage();

        // Handle the Streaming issue
        InputStreamForker inMessageAsStream = PatternHelper.handleStreamSource(inMessage);
        // send all exchanges asynchronously
        for (Consumes consumedService : consumesList) {
            if (inMessageAsStream != null) {
                // Handle the Streaming issue
                Source inSource = new StreamSource(inMessageAsStream.getInputStreamOne());
                inMessageAsStream = new InputStreamForker(inMessageAsStream.getInputStreamTwo());
                inMessage.setContent(inSource);
            }

            MEPConstants mep = PatternHelper.retrievePattern(consumedService.getMep());
            if (mep == null) {
                mep = MEPConstants.valueOf(exchange.getPattern());
            }
            Exchange dipatchedExchange = PatternHelper.createExchange(consumedService, mep,
                    inMessage, context);
            if (dipatchedExchange.getOperation() == null) {
                dipatchedExchange.setOperation(exchange.getOperation());
            }
            logSend(consumedService, dipatchedExchange);
            context.sendAsync(dipatchedExchange);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.Pattern#init()
     */
    public void init() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractPattern#getPatternName()
     */
    @Override
    protected String getPatternName() {
        return "Dispatcher";
    }
}
