/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.se.eip.patterns;

import java.net.URI;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.dom.DOMSource;
import javax.xml.xpath.XPathExpressionException;

import org.ow2.petals.component.framework.Constants.MEPConstants;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.se.eip.ExchangeContext;
import org.w3c.dom.Document;

/**
 * A Router pattern
 * 
 * @author alouis
 * @author ofabre
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since 2.2
 * 
 */
public class Router extends AbstractPattern {

    public static final String CONDITION_PREFIX = "router-condition";

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.se.eip.patterns.AbstractPattern#process(org.ow2.petals
     * .component.framework.util.Exchange,
     * org.ow2.petals.se.eip.ExchangeContext)
     */
    @Override
    public void process(Exchange exchange, ExchangeContext context) throws MessagingException,
            PEtALSCDKException {
        List<Consumes> consumesList = context.getSUConsumes(exchange.getEndpoint());

        Map<Integer, String> conditions = retrieveConditions(context.getExtensions());

        // there must be numberOfConditions+1' consumes' defined in the SU
        int count = consumesList.size();
        if (!(count == conditions.size() + 1)) {
            throw new PEtALSCDKException(
                    "The Service Unit must define (numberOfConditions+1) Consumes sections for the pattern "
                            + this.getPatternName());
        }

        Exchange originalExchange = exchange;
        Document inMessage = originalExchange.getInMessageContentAsDocument();
        // Retrieve identifier of the first consumes linked to a true
        // condition, or the default one
        int targetedConsumesListIndentifier;
        try {
            targetedConsumesListIndentifier = retrieveTargetedConsumesListIdentifier(conditions,
                    inMessage, context);
        } catch (XPathExpressionException e) {
            throw new PEtALSCDKException(e);
        }

        // forward the exchange to the correct service, depending on the
        // test result
        Consumes consumedService = consumesList.get(targetedConsumesListIndentifier);
        // reset to content to avoid streaming issue
        originalExchange.getInMessage().setContent(new DOMSource(inMessage));
        MEPConstants mep = PatternHelper.retrievePattern(consumedService.getMep());
        if (mep == null) {
            mep = MEPConstants.valueOf(exchange.getPattern());
        }
        Exchange routedExchange = PatternHelper.createExchange(consumedService, mep,
                originalExchange.getInMessage(), context);
        if (routedExchange.getOperation() == null) {
            routedExchange.setOperation(originalExchange.getOperation());
        }

        logSend(consumedService, routedExchange);
        context.sendSync(routedExchange);

        // process the response
        NormalizedMessage receivedMessage = PatternHelper.processSentExchange(routedExchange,
                originalExchange, context);
        // if a response has been returned, set it on the original
        // exchange
        // otherwise, the' processSentExchange()' has manage the
        // exchange
        if (receivedMessage != null) {
            originalExchange.setOutMessage(receivedMessage);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractPattern#getPatternName()
     */
    @Override
    protected String getPatternName() {
        return "Router";
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.se.eip.patterns.AbstractPattern#validateMEP(java.net.URI)
     */
    @Override
    protected boolean validateMEP(URI mep) {
        return MEPConstants.IN_ONLY_PATTERN.equals(mep)
                || MEPConstants.ROBUST_IN_ONLY_PATTERN.equals(mep)
                || MEPConstants.IN_OUT_PATTERN.equals(mep);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.Pattern#init()
     */
    public void init() {
    }

    /**
     * 
     * @param conditions
     * @param inMessage
     * @param context
     * @return
     * @throws XPathExpressionException
     */
    private int retrieveTargetedConsumesListIdentifier(Map<Integer, String> conditions,
            Document inMessage, ExchangeContext context) throws XPathExpressionException {
        int targetedConsumesListIndentifier = conditions.size();
        for (int i = 1; i <= conditions.size(); i++) {
            // evaluate the IN message
            logger.info("Evaluate condition # " + CONDITION_PREFIX + i + " : " + conditions.get(i));
            if (PatternHelper.evaluateXPathCondition(conditions.get(i), inMessage.getFirstChild(),
                    context)) {
                targetedConsumesListIndentifier = i - 1;
                break;
            }
        }
        return targetedConsumesListIndentifier;
    }

    /**
     * 
     * @param extensions
     * @return
     */
    private Map<Integer, String> retrieveConditions(ConfigurationExtensions extensions) {
        Map<Integer, String> conditions = new HashMap<Integer, String>();

        for (String param : extensions.keySet()) {
            if (param.equals(CONDITION_PREFIX)) {
                // only one parameter
                conditions.put(new Integer(1), extensions.get(param));
            } else if (param.startsWith(CONDITION_PREFIX)) {
                conditions.put(Integer.valueOf(param.substring(CONDITION_PREFIX.length())),
                        extensions.get(param));
            }
        }

        return conditions;
    }
}
