/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.se.eip.patterns;

import java.net.URI;
import java.util.List;
import java.util.logging.Level;

import javax.jbi.JBIException;
import javax.jbi.messaging.NormalizedMessage;

import org.ow2.petals.component.framework.Constants.MEPConstants;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.se.eip.ExchangeContext;

/**
 * Pipe all Consumes services defined in the service unit.<br>
 * - If InOut, return the last result. <br>
 * - If In, exchange is closed. <br>
 * - If a Fault or Error occurs on a call, stop the process and return the Fault
 * or Error
 * 
 * @author alouis, chamerling - eBM WebSourcing
 * 
 */
public class RoutingSlip extends AbstractPattern {

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.se.eip.patterns.AbstractPattern#process(org.ow2.petals
     * .component.framework.util.Exchange,
     * org.ow2.petals.se.eip.ExchangeContext)
     */
    public void process(Exchange exchange, ExchangeContext context) {

        Exchange originalExchange = exchange;

        List<Consumes> consumesList = context.getSUConsumes(originalExchange.getEndpoint());
        // assumes all services to chain are based on the IN-OUT MEP
        // but the last call follow the MEP specified in the PROVIDES
        // section of the SU
        int count = consumesList.size();
        if (count >= 1) {
            try {
                NormalizedMessage receivedMessage = originalExchange.getInMessage();
                for (int i = 0; i < count && receivedMessage != null; i++) {
                    // For all calls, use InOut M.E.P.
                    // For the last call, use the original M.E.P.
                    MEPConstants mep = (i < count - 1) ? MEPConstants.IN_OUT_PATTERN : MEPConstants
                            .valueOf(originalExchange.getPattern());

                    // create the exchange to pipe to the next service
                    Consumes consumes = consumesList.get(i);
                    Exchange pipedExchange = PatternHelper.createExchange(consumes, mep,
                            receivedMessage, context);

                    // send the exchange to the next service
                    logSend(consumes, exchange);
                    context.sendSync(pipedExchange);

                    // process the response
                    receivedMessage = PatternHelper.processSentExchange(pipedExchange,
                            originalExchange, context);

                    // return if there is a fault
                    if (originalExchange.getFault() != null) {
                        return;
                    }
                }
                // if the last piped service return a response, send it back
                // otherwise, the' processSentExchange()' has set information
                // on the original exchange
                if (receivedMessage != null) {
                    originalExchange.setOutMessage(receivedMessage);
                }
            } catch (JBIException e) {
                logger.log(Level.SEVERE, e.getMessage(), e);
                exchange.setError(e);
            }
        } else {
            Exception exception = new PEtALSCDKException(
                    "The Service Unit must define at least 1 Consumes sections for the pattern "
                            + this.getPatternName());
            logger.log(Level.SEVERE, exception.getMessage(), exception);
            exchange.setError(exception);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.Pattern#init()
     */
    public void init() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractPattern#getPatternName()
     */
    @Override
    protected String getPatternName() {
        return "RoutingSlip";
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.se.eip.patterns.AbstractPattern#validateMEP(java.net.URI)
     */
    @Override
    protected boolean validateMEP(URI mep) {
        return true;
    }

}
